@@ -27,6 +27,7 @@ from covsirphy.visualization.colored_map import ColoredMap
 from covsirphy.visualization.japan_map import jpn_map
 from covsirphy.visualization.line_plot import LinePlot, line_plot
 from covsirphy.visualization.bar_plot import BarPlot, bar_plot
+from covsirphy.visualization.compare_plot import ComparePlot, compare_plot
 # cleaning
 from covsirphy.cleaning.cbase import CleaningBase
 from covsirphy.cleaning.jhu_data import JHUData
@@ -51,6 +52,7 @@ from covsirphy.ode.sird import SIRD
 from covsirphy.ode.sirf import SIRF
 from covsirphy.ode.sirfv import SIRFV
 from covsirphy.ode.sewirf import SEWIRF
+from covsirphy.ode.ode_handler import ODEHandler
 # simulation
 from covsirphy.simulation.estimator import Estimator, Optimizer
 from covsirphy.simulation.simulator import ODESimulator
@@ -64,6 +66,7 @@ from covsirphy.regression.reg_handler import RegressionHandler
 from covsirphy.analysis.example_data import ExampleData
 from covsirphy.analysis.data_handler import DataHandler
 from covsirphy.analysis.param_tracker import ParamTracker
+from covsirphy.analysis.phase_tracker import PhaseTracker
 from covsirphy.analysis.scenario import Scenario
 from covsirphy.analysis.model_validator import ModelValidator
 # worldwide
@@ -98,6 +101,7 @@ __all__ = [
     "UnExpectedValueError", "NotRegisteredMainError", "NotRegisteredExtraError",
     # visualization
     "VisualizeBase", "ColoredMap", "LinePlot", "line_plot", "BarPlot", "bar_plot",
+    "ComparePlot", "compare_plot",
     # cleaning
     "CleaningBase", "DataLoader", "COVID19DataHub",
     "JHUData", "CountryData", "PopulationData", "OxCGRTData", "VaccineData",
@@ -105,20 +109,15 @@ __all__ = [
     # trend
     "TrendDetector", "TrendPlot", "trend_plot",
     # ode
-    "ModelBase", "SIR", "SIRD", "SIRF", "SEWIRF",
-    # simulation
-    "ODESimulator", "Estimator",
-    # phase
-    "PhaseSeries", "PhaseUnit", "MPEstimator",
+    "ModelBase", "SIR", "SIRD", "SIRF", "SEWIRF", "ODEHandler",
     # regression
     "RegressionHandler",
     # analysis
-    "ExampleData", "Scenario", "ModelValidator", "ParamTracker", "DataHandler",
-    # worldwide
-    "PolicyMeasures",
+    "ExampleData", "Scenario", "ModelValidator", "DataHandler", "PhaseTracker",
     # Deprecated
     "Population", "Word", "jpn_map", "SIRFV", "line_plot_multiple", "ChangeFinder", "Trend",
-    "Optimizer", "save_dataframe",
+    "Optimizer", "save_dataframe", "PolicyMeasures", "ODESimulator", "Estimator", "ParamTracker",
+    "PhaseSeries", "PhaseUnit", "MPEstimator",
 ]
 
 # Show exceptions in better format if used from command line
