@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 
 import numpy as np
+from covsirphy.util.error import deprecate
 from covsirphy.ode.mbase import ModelBase
 
 
@@ -21,11 +22,11 @@ class SEWIRF(ModelBase):
     # Variable names in (non-dim, dimensional) ODEs
     VAR_DICT = {
         "x1": ModelBase.S,
+        "x2": ModelBase.E,
+        "x3": ModelBase.W,
         "y": ModelBase.CI,
         "z": ModelBase.R,
         "w": ModelBase.F,
-        "x2": ModelBase.E,
-        "x3": ModelBase.W,
     }
     VARIABLES = list(VAR_DICT.values())
     # Weights of variables in parameter estimation error function
@@ -41,7 +42,7 @@ class SEWIRF(ModelBase):
             "rho2": 0.167, "rho3": 0.167,
         },
         ModelBase.Y0_DICT: {
-            ModelBase.S: 994_000, ModelBase.E: 3000, ModelBase.W: 0.002,
+            ModelBase.S: 994_000, ModelBase.E: 3000, ModelBase.W: 2000,
             ModelBase.CI: 1000, ModelBase.R: 0, ModelBase.F: 0,
         },
     }
@@ -60,9 +61,7 @@ class SEWIRF(ModelBase):
             - @sigma (float)
         """
         # Total population
-        self.population = self._ensure_natural_int(
-            population, name="population"
-        )
+        self.population = self._ensure_population(population)
         # Non-dim parameters
         self.theta = theta
         self.kappa = kappa
@@ -99,9 +98,10 @@ class SEWIRF(ModelBase):
         return np.array([dsdt, didt, drdt, dfdt, dedt, dwdt])
 
     @classmethod
+    @deprecate(".param_range()", new=".guess()", version="2.19.1-zeta-fu1")
     def param_range(cls, taufree_df, population):
         """
-        Define the range of parameters (not including tau value).
+        Deprecated. Define the range of parameters (not including tau value).
 
         Args:
             taufree_df (pandas.DataFrame):
@@ -123,9 +123,10 @@ class SEWIRF(ModelBase):
         )
 
     @classmethod
+    @deprecate(".specialize()", new=".convert()", version="2.19.1-zeta-fu1")
     def specialize(cls, data_df, population):
         """
-        Specialize the dataset for this model.
+        Deprecated. Specialize the dataset for this model.
 
         Args:
             data_df (pandas.DataFrame):
@@ -149,8 +150,8 @@ class SEWIRF(ModelBase):
                     - Exposed (int): 0
                     - Waiting (int): 0
         """
-        df = cls._ensure_dataframe(
-            data_df, name="data_df", columns=cls.VALUE_COLUMNS)
+        cls._ensure_dataframe(data_df, name="data_df", columns=cls.VALUE_COLUMNS)
+        df = data_df.copy()
         # Calculate dimensional variables
         df[cls.S] = population - df[cls.C]
         df[cls.E] = 0
@@ -158,9 +159,10 @@ class SEWIRF(ModelBase):
         return df
 
     @classmethod
+    @deprecate(".restore()", new=".convert_reverse()", version="2.19.1-zeta-fu1")
     def restore(cls, specialized_df):
         """
-        Restore Confirmed/Infected/Recovered/Fatal.
+        Deprecated. Restore Confirmed/Infected/Recovered/Fatal.
          using a dataframe with the variables of the model.
 
         Args:
@@ -228,3 +230,100 @@ class SEWIRF(ModelBase):
             }
         except ZeroDivisionError:
             return {p: None for p in self.DAY_PARAMETERS}
+
+    @classmethod
+    def convert(cls, data, tau):
+        """
+        Divide dates by tau value [min] and convert variables to model-specialized variables.
+
+        Args:
+            data (pandas.DataFrame):
+                Index
+                    reset index
+                Columns
+                    - Date (pd.Timestamp): Observation date
+                    - Susceptible(int): the number of susceptible cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal(int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
+            tau (int): tau value [min] or None (skip division by tau values)
+
+        Returns:
+            pandas.DataFrame:
+                Index
+                    - Date (pd.Timestamp): Observation date (available when @tau is None)
+                    - t (int): time steps (available when @tau is not None)
+                Columns
+                    - Susceptible (int): the number of susceptible cases
+                    - Exposed (int): 0
+                    - Waiting (int): 0
+                    - Infected (int): the number of currently infected cases
+                    - Recovered (int): the number of recovered cases
+                    - Fatal (int): the number of fatal cases
+        """
+        # Convert to tau-free if tau was specified
+        df = cls._convert(data, tau)
+        # Conversion of variables
+        df[cls.E] = 0
+        df[cls.W] = 0
+        return df.loc[:, [cls.S, cls.E, cls.W, cls.CI, cls.R, cls.F]]
+
+    @classmethod
+    def convert_reverse(cls, converted_df, start, tau):
+        """
+        Calculate date with tau and start date, and restore Susceptible/Infected/Fatal/Recovered.
+
+        Args:
+            converted_df (pandas.DataFrame):
+                Index
+                    time steps: Dates divided by tau value
+                Columns
+                    - Susceptible (int): the number of susceptible cases
+                    - Exposed (int): exposed and in latent period (without infectivity)
+                    - Waiting (int): waiting for confirmaion diagnosis (with infectivity)
+                    - Infected (int): the number of currently infected cases
+                    - Recovered (int): the number of recovered cases
+                    - Fatal (int): the number of fatal cases
+            start (pd.Timestamp): start date of simulation, like 14Apr2021
+            tau (int): tau value [min]
+
+        Returns:
+            pandas.DataFrame:
+                Index
+                    reset index
+                Columns
+                    - Date (pd.Timestamp): Observation date
+                    - Susceptible(int): the number of susceptible cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal(int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
+        """
+        # Calculate date with tau and start date
+        df = cls._convert_reverse(converted_df, start, tau)
+        # Conversion of variables
+        df[cls.S] = df[cls.S] + df[cls.E] + df[cls.W]
+        return df.loc[:, [cls.DATE, cls.S, cls.CI, cls.F, cls.R]]
+
+    @classmethod
+    def guess(cls, data, tau, q=0.5):
+        """
+        With (X, dX/dt) for X=S, I, R and so on, guess parameter values.
+        This is not implemented for SEWIR-F model.
+
+        Args:
+            data (pandas.DataFrame):
+                Index
+                    reset index
+                Columns
+                    - Date (pd.Timestamp): Observation date
+                    - Susceptible(int): the number of susceptible cases
+                    - Infected (int): the number of currently infected cases
+                    - Fatal(int): the number of fatal cases
+                    - Recovered (int): the number of recovered cases
+            tau (int): tau value [min]
+            q (float or tuple(float,)): the quantile(s) to compute, value(s) between (0, 1)
+        """
+        raise NotImplementedError(
+            "SEWIR-F cannot be used for parameter estimation because we do not have records "
+            "of Exposed and Waiting. Please use SIR-F model with `covsirphy.SIRF` class."
+        )
